SystemOrganization addCategory: #'File-Kernel'!

!String methodsFor: '*file-kernel' stamp: 'kph 11/13/2008 00:36'!
asDirectory

	^ FileKernel newDirectoryFrom: self! !

!String methodsFor: '*file-kernel' stamp: 'kph 11/13/2008 00:35'!
asFile

	^ FileKernel newFileFrom: self! !

Object subclass: #FileExecutive
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'File-Kernel'!

!FileExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/23/2008 23:41'!
OSProcessOrNil

	"extra check is needed because OSProcess on windows does not support waitForCommand (yet)"
	
	| osp |
	
	osp := Smalltalk at: #OSProcess ifAbsent: [ ^ nil ].
	
	^ (osp thisOSProcess respondsTo: #waitForCommand:) ifTrue: [ osp ] ifFalse: [ nil ]
	! !

!FileExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/12/2008 23:46'!
canInstanciateFrom: aPathString

	"we return false, because we do not support any specific protocol"

	^ false! !

!FileExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/12/2008 19:34'!
default

	^ FileLocalExecutive            ! !

!FileExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 01:27'!
dirClass

	^ Smalltalk at: #Directory ifAbsent: [ FileKernel ]! !

!FileExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 01:28'!
fileClass

	^ Smalltalk at: #File ifAbsent: [ FileKernel ]! !

!FileExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/6/2008 14:44'!
fileStreamClass

	^ FileStream concreteStream! !

!FileExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 00:32'!
instanciatorFrom: aString

	| executiveClass |
	
	FileExecutive allSubclassesDo: [ :c | (c canInstanciateFrom: aString) ifTrue: [ executiveClass := c ]].
	
	^ executiveClass ifNil: [ self default ]! !

!FileExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 01:26'!
newDirectoryFrom: aString

	^ (self instanciatorFrom: aString) makeNew: self dirClass from: aString
    ! !

!FileExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 01:27'!
newFileFrom: aString

	^ (self instanciatorFrom: aString) makeNew: self fileClass from: aString
    ! !

!FileExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/9/2008 19:12'!
pathDelimiter
 
	^ '/'
! !

!FileExecutive methodsFor: 'case-sensitivity' stamp: 'kph 11/9/2008 17:47'!
MCPasswordManager

	^ MCPasswordManager! !

!FileExecutive methodsFor: 'local/remote file copy' stamp: 'kph 11/24/2008 22:52'!
addAll: someFD relativeTo: aBaseDir toDir: aDir
	
	^ aDir executive toDir: aDir addAllLocal: someFD relativeTo: aBaseDir.! !

!FileExecutive methodsFor: 'case-sensitivity' stamp: 'kph 11/3/2008 01:37'!
arePathsCaseSensitive

	^ (self is: $a sameAs: $A) not
	
! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 01:52'!
basicReader: aFile

	^ self class fileStreamClass new open: aFile forWrite: false! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/6/2008 14:49'!
basicWriter: aRio

	^ self class fileStreamClass new open: aRio forWrite: true 
	! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'test 11/12/2008 23:24'!
delete: aFileOrDir

	aFileOrDir isDirectory ifTrue: [ ^ aFileOrDir rmdir ].
	self deleteFile: aFileOrDir.! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/25/2008 03:42'!
dir: aDir mkpath: aFileOrDir relativeTo: aBaseDirectory
	
	| newFD |
	
	newFD := aBaseDirectory ifNil: [ aFileOrDir ]
				    		 	 ifNotNil: [ aDir / (aFileOrDir linearRelativeTo: aBaseDirectory) ].

 	aFileOrDir isDirectory ifTrue: [ newFD mkpath ]. 
	aFileOrDir isFile ifTrue: [ newFD parent mkpath ].
	
	^ newFD
	
 ! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/12/2008 20:28'!
dirClass

	^ Smalltalk at: #Directory ifAbsent: [ FileKernel ]! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/12/2008 20:26'!
fileClass

	^ Smalltalk at: #File ifAbsent: [ FileKernel ]! !

!FileExecutive methodsFor: 'testing' stamp: 'kph 11/7/2008 03:56'!
fileSize: aRio

	^ aRio stat fileSize! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 00:15'!
fullFor: aFileOrDirectory

	"the absolute path of this rio, or if relative 
	combine with #defaultDirectory to obtain the full path"
	
	aFileOrDirectory value isEmpty ifTrue: [ ^ self defaultDirectory ].
	
	(self isFull: aFileOrDirectory) ifTrue: [ ^ aFileOrDirectory ].

	^ aFileOrDirectory newFrom: (self defaultDirectory value, self pathDelimiter, aFileOrDirectory value)! !

!FileExecutive methodsFor: 'equality' stamp: 'kph 11/3/2008 01:39'!
hashFor: aRio
	
	^ aRio value hash! !

!FileExecutive methodsFor: 'executive actions' stamp: 'kph 11/3/2008 01:41'!
importValue: path
 
	path isEmpty ifTrue: [  ^ '' ].
	
	(self isRoot: path) ifTrue: [ ^ self rootString ].
	
 	(path last = self pathDelimiter first) ifTrue: [ ^ path allButLast ].
	
	^path! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'test 11/12/2008 23:29'!
in: aDir select: selectBlock

	^ self 
		startAt: aDir
		recursively: aDir isRecursive 
		select: selectBlock 
		into: (OrderedCollection new: 50)
! !

!FileExecutive methodsFor: 'equality' stamp: 'kph 11/3/2008 01:37'!
is: aRio sameAs: bRio

	^ aRio value = bRio value! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'test 11/3/2008 01:47'!
isDirectory: aRio 

	^ aRio stat ifNil: [ false ] ifNotNil: [ aRio stat isDirectory ]
! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 00:10'!
isFile: aFileOrDirectory 

	^ (aFileOrDirectory stat ifNil: [ ^ false ]) isFile
! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 00:15'!
isFull: aFileOrDirectory

	^ aFileOrDirectory value beginsWith: self rootString! !

!FileExecutive methodsFor: 'testing' stamp: 'kph 11/3/2008 01:35'!
isRoot: aRioOrString

	^ aRioOrString value = self rootString! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/12/2008 20:26'!
makeNewRioFromString: aString

	^ self fileClass executive: self value: aString ! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/6/2008 20:32'!
mkdir: aRio

	aRio isDirectory ifTrue: [ ^ self error: 'directory already exists' ].
	self createDirectory: aRio  ! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/6/2008 19:42'!
mkpath: aRio

	aRio isDirectory ifTrue: [ ^self ].
	aRio parent mkpath.
	self createDirectory: aRio.  ! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/3/2008 01:43'!
pathDelimiter
 
	^ '/'
! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/3/2008 01:43'!
pathDelimiterChar
 
	^ $/
! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 01:34'!
root

	^ self class makeNew: self class dirClass from: self rootString! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/3/2008 01:36'!
rootString

	^ self subclassResponsibility! !

!FileExecutive methodsFor: 'local/remote file copy' stamp: 'kph 11/24/2008 22:53'!
toDir: aDir addAllLocal: someFD relativeTo: aBaseDir

	^ self toDirLocal: aDir addAllLocal: someFD relativeTo: aBaseDir.! !

!FileExecutive methodsFor: 'local/remote file copy' stamp: 'kph 11/25/2008 04:34'!
toDir: aDir addAllRemote: someFD relativeTo: aBaseDir

	^ someFD first executive toDirLocal: aDir addAllRemote: someFD relativeTo: aBaseDir.! !

!FileExecutive methodsFor: 'local/remote file copy' stamp: 'kph 11/25/2008 05:09'!
toDir: aDir mkpathAll: someFD relativeTo: aBaseDir

	"take the list of files and ensure that the directories exist, return sourceFile -> destFile map"

	^ someFD 
		collect: [ :aFileOrDir | Array with: aFileOrDir with: (self dir: aDir mkpath: aFileOrDir relativeTo: aBaseDir) ]
		thenSelect: [ :ea | ea first isFile ].
	
	 ! !

!FileExecutive methodsFor: 'local/remote file copy' stamp: 'test 11/25/2008 05:07'!
toDirLocal: aDir addAllLocal: someFD relativeTo: aBaseDir

	| map |

	map := self toDir: aDir mkpathAll: someFD relativeTo: aBaseDir.

	map do: [ :each | self copyLocalFile: each first toLocalFile: each second ].
	
	^ map! !

!FileExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/6/2008 13:21'!
touch: aRio

	aRio writer close! !

FileExecutive subclass: #FileLocalExecutive
	instanceVariableNames: 'rootString root defaultDirectory newRio'
	classVariableNames: 'Current Test'
	poolDictionaries: ''
	category: 'File-Kernel'!

!FileLocalExecutive commentStamp: 'kph 4/17/2007 19:25' prior: 0!
Introducing the idea that Rio's exist in the context of their 'physical container', we have the RioLocalFileSystem as the primary container which provides the file system interface, also known as the 'executive'.

On unix there is one executive for the whole system. On dos there is one executive per volume. This approach also supports the notion of files within archives and files on remote servers.

On startup, having a formal model of the host filesystem allows us to keep a current and previous instance introducing the possibility for migrating Rio's on startup when the image has been moved. For example C:/ could be replaced with /mnt/hda1 thus attempting to make images properly portable. (experimental idea)

!

!FileLocalExecutive class methodsFor: 'class initialization' stamp: 'kph 12/2/2008 16:18'!
initialize

	Smalltalk addToStartUpList: self after: Delay.
	FileLocalDosExecutive initializeVolumes.
	Current := self newForThisPlatform.
	Current initializeDefault.
	! !

FileLocalExecutive subclass: #FileLocalCaseInsensitiveExecutive
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'File-Kernel'!

!FileLocalCaseInsensitiveExecutive methodsFor: 'case-sensitivity' stamp: 'kph 3/28/2007 18:41'!
hashFor: aRio
	
	^ aRio value asLowercase hash! !

!FileLocalCaseInsensitiveExecutive methodsFor: 'case-sensitivity' stamp: 'kph 4/4/2007 02:36'!
is: aRio sameAs: bRio

^ (aRio value compare: bRio value caseSensitive: false) = 2! !

FileLocalCaseInsensitiveExecutive subclass: #FileLocalDosExecutive
	instanceVariableNames: ''
	classVariableNames: 'Volumes'
	poolDictionaries: ''
	category: 'File-Kernel'!

!FileLocalDosExecutive commentStamp: 'kph 4/17/2007 19:19' prior: 0!
The initial idea was to model things properly as a file within a named volume, on unix there would be one executive for the whole system. On dos there would be one executive per volume.!

!FileLocalDosExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/12/2008 23:46'!
canInstanciateFrom: aString

	^ (aString size > 1 and: [ aString second = $: ])! !

!FileLocalDosExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/9/2008 02:17'!
executiveForVolume: theRequestedRoot
 
	| requestedRoot |
	requestedRoot := theRequestedRoot.
	requestedRoot = nil ifTrue: [ requestedRoot := 'C:/' ].
	
	^ Volumes 
		detect: [ :i | i isRoot: requestedRoot ] 
		ifNone: [ Volumes add: (self class basicNew setRootString: requestedRoot; setDefault: nil; yourself) ]! !

!FileLocalDosExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 4/3/2007 13:37'!
initializeVolumes 
	
	Volumes := Set new ! !

!FileLocalDosExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 01:26'!
makeNew: aClass from: str

	| rioDrive exec |
	
	rioDrive := nil.
	
	(str size > 1) ifTrue: [  

		(str second = $:) ifTrue: [ rioDrive := (str first: 2) , self pathDelimiter ].
		
	].
	exec := self executiveForVolume: rioDrive.

	^ aClass executive: exec value: str! !

!FileLocalDosExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/9/2008 19:11'!
pathDelimiter
 
	^ '\'
! !

!FileLocalDosExecutive methodsFor: 'accessing' stamp: 'kph 11/3/2008 01:35'!
drive 

	^ self rootString first: 2! !

!FileLocalDosExecutive methodsFor: 'accessing' stamp: 'kph 11/3/2008 01:35'!
driveLetter

	^ self rootString first! !

!FileLocalDosExecutive methodsFor: 'accessing' stamp: 'kph 11/3/2008 01:42'!
executiveForVolume: requestedRoot
 
	^ Volumes 
		detect: [ :i | i isRoot: requestedRoot ] 
		ifNone: [ Volumes add: (self class basicNew setRootString: requestedRoot; setDefault: nil; yourself) ]! !

!FileLocalDosExecutive methodsFor: 'as yet unclassified' stamp: 'kph 9/19/2007 16:28'!
fullFor: aRio

	"the absolute path of this rio (with the current volume drive prepended), or if relative 
	combine with #defaultDirectory to obtain the full path"
	
	aRio value isEmpty ifTrue: [ ^ self defaultDirectory ].
	
	(self isFull: aRio) ifTrue: [ ^ aRio ].
	
	"if we are an absolute path, prepend the volume"
	aRio value first = self pathDelimiterChar ifTrue: [ ^ aRio newFrom: (self drive, aRio value) ].

	^ aRio newFrom: (self defaultDirectory value, self pathDelimiter, aRio value)! !

!FileLocalDosExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/12/2008 20:28'!
getTempDirectory

 	^ self dirClass new: 'C:\WINDOWS\TEMP\'! !

!FileLocalDosExecutive methodsFor: 'as yet unclassified' stamp: 'test 3/30/2007 02:55'!
importValue: aPath

	^ super importValue: (aPath copy replaceAll: $/ with: $\).! !

!FileLocalDosExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/3/2008 01:42'!
initializeDefault
  
	self setRootString: (self primImagePath first: 3);
		setDefault: self getDefaultDirectory.

	Volumes add: self.! !

!FileLocalDosExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/3/2008 01:35'!
isRoot: aRioOrString

	aRioOrString value size = self rootString size ifTrue: [ ^ super isRoot: aRioOrString ].
	
	^ (aRioOrString value, self pathDelimiter) = self rootString! !

!FileLocalDosExecutive methodsFor: 'fs migration' stamp: 'kph 11/12/2008 23:47'!
makeNewFrom: aString

	| rioDrive exec str |
	
	str := aString.
	exec := self.
	
	(aString size > 1) ifTrue: [  

		rioDrive := rootString.
		(str second = $:) ifTrue: [ rioDrive := (str first: 2) , self pathDelimiter ].
		
		rioDrive ~= rootString ifTrue: [ exec := self executiveForVolume: rioDrive ].

	].

	^ exec fileClass executive: exec value: str! !

!FileLocalDosExecutive methodsFor: 'fs migration' stamp: 'kph 11/12/2008 20:26'!
makeNewRioFrom: aString

	| rioDrive exec str |
	
	str := aString.
	exec := self.
	
	(aString size > 1) ifTrue: [  

		rioDrive := rootString.
		(str second = $:) ifTrue: [ rioDrive := (str first: 2) , self pathDelimiter ].
		
		rioDrive ~= rootString ifTrue: [ exec := self executiveForVolume: rioDrive ].

	].

	^ exec fileClass executive: exec value: str! !

!FileLocalDosExecutive methodsFor: 'fs migration todo' stamp: 'kph 3/26/2007 05:49'!
migrateFromLocalDosFileSystem: nfs

  "could do something here"

 ^ nfs ! !

!FileLocalDosExecutive methodsFor: 'fs migration' stamp: 'kph 3/26/2007 05:54'!
migrateFromLocalFileSystem: nfs

 ^ self  ! !

!FileLocalDosExecutive methodsFor: 'fs migration' stamp: 'kph 3/26/2007 05:38'!
migrateToLocalDosFileSystem: old

 ^ self! !

!FileLocalDosExecutive methodsFor: 'as yet unclassified' stamp: 'kph 3/30/2007 03:03'!
pathDelimiter
 
	^ '\'
! !

!FileLocalDosExecutive methodsFor: 'as yet unclassified' stamp: 'kph 3/31/2007 02:59'!
pathDelimiterChar
 
	^ $\
! !

!FileLocalExecutive class methodsFor: 'accessing current' stamp: 'kph 3/30/2007 00:34'!
current

	^ Test ifNil: [ Current ]
 	
! !

!FileLocalExecutive class methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 01:32'!
makeNew: aClass from: aString

	^ aClass executive: self current value: aString ! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 3/28/2007 21:54'!
maxFileNameLengthOnDOS

	^ 8! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 3/28/2007 19:31'!
maxFileNameLengthOnMacOS

	^ 31! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 3/28/2007 21:55'!
maxFileNameLengthOnRiscOS

^ 255! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 3/28/2007 19:31'!
maxFileNameLengthOnUnix

^ 255! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 3/28/2007 19:31'!
maxFileNameLengthOnWin32

	^ 255! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 11/12/2008 19:33'!
newForDOS

	^ FileLocalDosExecutive new! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 11/12/2008 19:33'!
newForMacOS

	^ FileLocalCaseInsensitiveExecutive new! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 11/12/2008 19:34'!
newForRiscOS

	^ FileLocalExecutive new! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 11/9/2008 19:46'!
newForThisPlatform

	^ self perform: ('newFor', self platform) asSymbol! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 12/2/2008 16:18'!
newForUnix

	^ FileLocalExecutive new! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 12/2/2008 16:18'!
newForWin32

	^ FileLocalDosExecutive new! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 4/11/2007 21:15'!
performStartUpMigration
 	| old |
	
	old := self current.
	Current := self newForThisPlatform.
	Current initializeDefault.

	old migrateFileSystemTo: Current! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 3/28/2007 19:26'!
platform

	^ SmalltalkImage current platformName asLegalSelector capitalized! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 4/4/2008 21:10'!
startUp: resuming
	
 	resuming ifFalse: [ ^self ].
	 		
	self performStartUpMigration.! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 3/30/2007 00:31'!
testEnd

	Test := nil! !

!FileLocalExecutive class methodsFor: 'image start up ' stamp: 'kph 4/3/2007 14:06'!
testFileSystem: fs

	^ Test := fs! !

!FileLocalExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/24/2008 01:45'!
copyFile: aFile toLocalFile: bFile
	
	^ self copyLocalFile: aFile toLocalFile: bFile
	! !

!FileLocalExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/24/2008 01:45'!
copyFile: aFile toRemoteFile: bFile

	^ bFile executive copyLocalFile: aFile toRemoteFile: bFile! !

!FileLocalExecutive methodsFor: 'as yet unclassified' stamp: 'kph 11/24/2008 02:28'!
copyLocalFile: aFile toLocalFile: bFile

	self class OSProcessOrNil ifNotNilDo: [ :osp | osp waitForCommand: 'cp "', aFile asVmPathName, '" "', bFile asVmPathName, '"'. ^ aFile].

	(self primCopyFile: aFile asVmPathName to: bFile asVmPathName) = #failed ifTrue: [ ^ aFile copyTo: bFile ].

	^ aFile

! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 5/4/2007 01:49'!
createDirectory: aRio
	 
	^ self primCreateDirectory: aRio asVmPathName  ! !

!FileLocalExecutive methodsFor: 'accessing' stamp: 'kph 3/26/2007 01:59'!
defaultDirectory

	^ defaultDirectory! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'test 11/12/2008 23:26'!
deleteDirectory: aDir
	 
	self primDeleteDirectory: aDir asVmPathName.
	aDir statIsNowInvalid! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 3/27/2007 22:51'!
deleteFile: aRio
	
	"Delete the file of the given name if it exists.
	If the first deletion attempt fails do a GC to force finalization of any lost references. ar 3/21/98 17:53"
		 
	self retryWithGC: [
		self primDeleteFileNamed: aRio asVmPathName ]
		until:[:result| result notNil]
		forFileNamed: aRio.
		
	aRio statIsNowInvalid! !

!FileLocalExecutive methodsFor: 'errors' stamp: 'kph 4/3/2007 13:09'!
errorMustBeInSameFileSystem: aDirectory

	^ self error: 'Must be in same file system', aDirectory, ',', self printString! !

!FileLocalExecutive methodsFor: 'errors' stamp: 'kph 11/12/2008 20:15'!
errorUnableToDeleteDir: fullPath

	| entries |
	
	entries := fullPath asDirectory entries.
		
	^ self error: 'unable to delete directory: ', fullPath, ((entries size >0) ifTrue: [' (not empty)'] ifFalse: ['']).
! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 11/10/2008 05:16'!
getDefaultDirectory

	^ (Preferences startInUntrustedDirectory 
					ifTrue: [ self untrustedDirectory mkpath ]
 					ifFalse: [ self getImageDirectory ] )! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 11/12/2008 20:29'!
getImageDirectory

 	^ (self dirClass executive: self value: self primImagePath) parent ! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 3/27/2007 20:42'!
getMacFileTypeAndCreator: aRio 

	| results typeString creatorString |
	"get the Macintosh file type and creator info for the file with the given name. Fails if the file does not exist or if the type and creator type arguments are not strings of length 4. Does nothing on other platforms (where the underlying primitive is a noop)."
	"FileDirectory default getMacFileNamed: 'foo'"

	typeString := ByteArray new: 4 withAll: ($? asInteger). 
	creatorString := ByteArray new: 4 withAll: ($? asInteger).
	[self primGetMacFileNamed: aRio asVmPathName
		type: typeString
		creator: creatorString.] ensure: 
		[typeString := typeString asString. 
		creatorString := creatorString asString].
	results := Array with: typeString convertFromSystemString with: creatorString convertFromSystemString.
	^results
! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 11/12/2008 20:29'!
getTempDirectory

 	^ self dirClass new: '/tmp'! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 11/3/2008 01:42'!
initializeDefault

	self setRootString: self pathDelimiter;
		setDefault: self getDefaultDirectory! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 4/11/2007 06:06'!
isDirectory: aRio
 
	^ #badDirectoryPath ~= (self primLookupEntryIn: aRio full value index: 1)
	 ! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 4/3/2007 13:08'!
linearRelativeTo: aDirectory

	self ~= aDirectory executive ifTrue: [ ^ self errorMustBeInSameFileSystem: aDirectory ].
	
	^ self root! !

!FileLocalExecutive methodsFor: 'misc' stamp: 'kph 3/28/2007 19:30'!
maxFileNameLength

 ^ (self class respondsTo: #maxFileNameLength) 
	ifTrue: [ self class maxFileNameLength ]
	ifFalse: [ self class perform: ('maxFileNameLengthOn', self class platform) asSymbol ]! !

!FileLocalExecutive methodsFor: 'fs migration' stamp: 'kph 3/26/2007 05:23'!
migrateFileSystemTo: next

	^ next migrateFromLocalFileSystem: self ! !

!FileLocalExecutive methodsFor: 'fs migration' stamp: 'kph 3/26/2007 05:25'!
migrateFromLocalDosFileSystem: nfs

  "could do something here"

 ^ self ! !

!FileLocalExecutive methodsFor: 'fs migration' stamp: 'kph 3/26/2007 05:23'!
migrateFromLocalFileSystem: nfs

 ^ nfs ! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 11/24/2008 02:17'!
primCopyFile: srcName to: dstName 
	"Copied from VMMaker code.
	This really ought to be a facility in file system. The major annoyance 
	here is that file types and permissions are not handled by current 
	Squeak code.
	NOTE that this will clobber the destination file!!"
	<primitive: 'primitiveFileCopyNamedTo' module:'FileCopyPlugin'> "primitiveExternalCall" 

	^ #failed! !

!FileLocalExecutive methodsFor: 'primitives' stamp: 'kph 3/27/2007 22:18'!
primCreateDirectory: fullPath
	"Create a directory named by the given path. Fail if the path is bad or if a file or directory by that name already exists."

 	<primitive: 'primitiveDirectoryCreate' module: 'FilePlugin'>
	self primitiveFailed! !

!FileLocalExecutive methodsFor: 'primitives' stamp: 'kph 3/27/2007 22:18'!
primDeleteDirectory: fullPath
	"Delete the directory named by the given path. Raise an error if the path is bad or if a directory by that name does not exist."

 	<primitive: 'primitiveDirectoryDelete' module: 'FilePlugin'>
	^ self errorUnableToDeleteDir: fullPath
! !

!FileLocalExecutive methodsFor: 'primitives' stamp: 'kph 3/27/2007 22:23'!
primDeleteFileNamed: aFileName
	"Delete the file of the given name. Return self if the primitive succeeds, nil otherwise."

	<primitive: 'primitiveFileDelete' module: 'FilePlugin'>
	^ nil
! !

!FileLocalExecutive methodsFor: 'primitives' stamp: 'kph 3/27/2007 20:40'!
primGetMacFileNamed: fileName type: typeString creator: creatorString
	"Get the Macintosh file type and creator info for the file with the given name. Fails if the file does not exist or if the type and creator type arguments are not strings of length 4. This primitive is Mac specific; it is a noop on other platforms."

 	<primitive: 'primitiveDirectoryGetMacTypeAndCreator' module: 'FilePlugin'>

! !

!FileLocalExecutive methodsFor: 'primitives' stamp: 'kph 3/29/2007 03:00'!
primImagePath
	"Answer the full path name for the current image."
	"self new primImagePath"

	<primitive: 121>
	self primitiveFailed! !

!FileLocalExecutive methodsFor: 'primitives' stamp: 'kph 3/27/2007 18:01'!
primLookupEntryIn: fullPath index: index

	"Look up the index-th entry of the directory with the given fully-qualified path (i.e., starting from the root of the file hierarchy) and return an array containing:

	<name> 
	<creationTime in seconds since squeak epoch> 
	<modificationTime>
	<dirFlag (dir is true)>
	<fileSize in bytes>

	The empty string enumerates the top-level files or drives. (For example, on Unix, the empty path enumerates the contents of '/'. On Macs and PCs, it enumerates the mounted volumes/drives.)

	Returns nil when index is past the end of the directory. 
	Fails if the given path is bad."

 	<primitive: 'primitiveDirectoryLookup' module: 'FilePlugin'>
	^ #badDirectoryPath

! !

!FileLocalExecutive methodsFor: 'primitives' stamp: 'kph 5/28/2008 15:56'!
primPathNameDelimiter
	"Return the path delimiter for the underlying platform's file system."

 	<primitive: 'primitiveDirectoryDelimitor' module: 'FilePlugin'>
	self primitiveFailed
! !

!FileLocalExecutive methodsFor: 'primitives' stamp: 'kph 3/24/2007 14:51'!
primRename: oldFileFullName to: newFileFullName 
	"Rename the file of the given name to the new name. Fail if there is no file of the old name or if there is an existing file with the new name.
	Changed to return nil instead of failing ar 3/21/98 18:04"

	<primitive: 'primitiveFileRename' module: 'FilePlugin'>
	^nil! !

!FileLocalExecutive methodsFor: 'primitives' stamp: 'kph 3/27/2007 20:40'!
primSetMacFileNamed: fileName type: typeString creator: creatorString
	"Set the Macintosh file type and creator info for the file with the given name. Fails if the file does not exist or if the type and creator type arguments are not strings of length 4. This primitive is Mac specific; it is a noop on other platforms."

 	<primitive: 'primitiveDirectorySetMacTypeAndCreator' module: 'FilePlugin'>
	self primitiveFailed
! !

!FileLocalExecutive methodsFor: 'primitives' stamp: 'kph 5/28/2008 15:54'!
primVmPath
	"Answer the path for the directory containing the Smalltalk virtual machine. Return the 	empty string if this primitive is not implemented."
	"Rio primVmPath"

	<primitive: 142>
	^ ''! !

!FileLocalExecutive methodsFor: 'printing' stamp: 'kph 11/3/2008 01:34'!
printId

	^ rootString! !

!FileLocalExecutive methodsFor: 'printing' stamp: 'kph 4/19/2007 01:45'!
printOn: str

	super printOn: str.
	str nextPut: $(; nextPutAll: self printId; nextPut: $).! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 3/24/2007 14:57'!
rename: aRio to: bRio
	
	(self retryWithGC:[
		self primRename: aRio asVmPathName to: bRio asVmPathName
	]
		until:[:result| result notNil]
		forFileNamed: aRio) ~~ nil ifTrue:[ aRio statIsNowInvalid. ^ bRio ].

	aRio exists ifFalse:[ ^ self error:'Attempt to rename a non-existent file:' , aRio].
	
	bRio exists ifTrue:[ ^self error: 'Failed to rename, ', bRio,' already exists.' ].! !

!FileLocalExecutive methodsFor: 'utilities' stamp: 'kph 3/24/2007 14:56'!
retryWithGC: execBlock until: okBlock forFileNamed: aRio

    ^ StandardFileStream 
		retryWithGC: execBlock
		until: okBlock
		forFileNamed: aRio! !

!FileLocalExecutive methodsFor: 'accessing' stamp: 'kph 11/3/2008 01:41'!
rootString

	^ rootString! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 3/27/2007 21:19'!
set: aRio macType: typeString creator: creatorString
 	"Set the Macintosh file type and creator info for the file with the given name. Fails if the file does not exist or if the type and creator type arguments are not strings of length 4."
	 
 	self primSetMacFileNamed: aRio asVmPathName
		type: typeString convertToSystemString
		creator: creatorString convertToSystemString.! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 4/3/2007 12:27'!
setDefault: d
 
	defaultDirectory := d! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 11/3/2008 01:41'!
setRootString: aString  
	rootString := aString.
	 ! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 11/12/2008 23:57'!
startAt: aDir recursively: beRecursive select: selectBlock into: results 

	"Return a collection of rio's selected by passing
	the directoryEntry array to the selectBlock.
	
	The KernelRio implemetation of setStatFromEntryArray: 
	does not populate these rios with stat data.
	Rio's more verbose implementation does.

	This can be called with startAt: aString, but if so beRecursive must be false.
		
	See primLookupEntryIn:index: for further details."

	| index entryArray entry isDir |

	index := 1.

	[ (entryArray := self primLookupEntryIn: aDir asVmPathName index: index) notNil ] 
		whileTrue: [
			#badDirectoryPath = entryArray ifTrue: [
				^ results "(InvalidDirectoryError pathName: aDir value) signal" ].

				isDir := entryArray at: 4.
				(entry := aDir / (entryArray at: 1)) 
									setStatFromDir: aDir andEntryArray:entryArray.
				
				isDir ifFalse: [  entry := entry asFile ].
				
				(selectBlock value: entry) ifTrue: [ results add: entry ].	
						
				(beRecursive and: [ isDir ]) 
					ifTrue: [ 
						self 
							startAt: entry
							recursively: beRecursive 
							select: selectBlock
							into: results   
					].	
			index := index + 1
		].
	
	^ results! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 11/12/2008 20:29'!
tempDirectory

 	^ self dirClass new: '/tmp'! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 11/24/2008 03:18'!
toUrl: aFD

	^ FileUrl new initializeFromPathString: (aFD full value replacing: $\ with: $/)! !

!FileLocalExecutive methodsFor: 'executive actions' stamp: 'kph 11/12/2008 20:29'!
untrustedDirectory

 	^ self dirClass new: (SecurityManager default primUntrustedUserDirectory)
	 ! !

Object subclass: #FileKernel
	instanceVariableNames: 'executive value stat'
	classVariableNames: 'TestFileSystem'
	poolDictionaries: ''
	category: 'File-Kernel'!

!FileKernel commentStamp: 'kph 11/2/2008 22:50' prior: 0!
The minimum Rio functionality required to support a kernel image
is implemented here, thus making the rest of rio unloadable.

Recursive querying via #select: is available however the selectBlock is passed a raw entryArray, rather than a RioStat instance. This is not intended to be used extensively but is needed as a minimum for #isFile/#isDirectory.
 
If either #reader or #writer should fail RioKernel returns a nil, 

A functionally complete interface to the parts of the fileName is available as follows:
getter - splitToBaseVersionAndExt: [ :b :v :e |  ]
setter - #base: <nil or newName> version: <nil or newVersion> ext: <nil or new ext>

Interface for file versions is available via:
#versions - returns a SortedCollection of the version numbers available for this rio
#latestVersion - returns a rio for the latest version of the current rio in the versions list
#nextVersion - returns a rio for the latest + 1 version of the current rio.

Rio's may be included in the concatenation of strings

RioKernel file copyTo: does a binary stream to stream copy, without a progress bar.
Rio's file copyTo: honours the setting of the Rio's binary mode, and displays a progress bar.

instVars
---------
value - internal (to Rio) holds the path!

!FileKernel class methodsFor: 'as yet unclassified' stamp: 'kph 3/30/2007 00:28'!
initialize

 TestFileSystem := nil! !

!FileKernel class methodsFor: 'as yet unclassified' stamp: 'kph 11/12/2008 19:32'!
defaultExecutive

	^ FileExecutive default! !

!FileKernel class methodsFor: 'as yet unclassified' stamp: 'kph 11/2/2008 23:06'!
executive: e value: v

	^ self basicNew 
			setExecutive: e
			value: v! !

!FileKernel class methodsFor: 'as yet unclassified' stamp: 'kph 4/17/2007 18:05'!
fileStreamClass

	^ FileStream concreteStream! !

!FileKernel class methodsFor: 'as yet unclassified' stamp: 'kph 11/2/2008 23:06'!
new

	^ self new: '' ! !

!FileKernel class methodsFor: 'as yet unclassified' stamp: 'kph 11/12/2008 23:50'!
new: aPathOrFile

	"the preferred generic instanciation method, 
	will call the correct specific instanciation method on the correct class"

	(aPathOrFile isKindOf: FileKernel) ifTrue: [ ^ aPathOrFile copy ].
	                                                                                                            	
	^ self defaultExecutive current class newDirectoryFrom: aPathOrFile! !

!FileKernel class methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 00:34'!
newDirectoryFrom: aPath
	                                                                                                            	
	^ self defaultExecutive current class newDirectoryFrom: aPath! !

!FileKernel class methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 00:34'!
newFileFrom: aPath
	                                                                                                            	
	^ self defaultExecutive current class newFileFrom: aPath! !

!FileKernel class methodsFor: 'as yet unclassified' stamp: 'kph 11/9/2008 02:53'!
thisImage

	^ self new: self defaultExecutive current primImagePath.! !

!FileKernel methodsFor: 'as yet unclassified' stamp: 'kph 11/12/2008 20:24'!
/ morePath

 	^ self newFrom: (self pathJoin: morePath)! !

!FileKernel methodsFor: 'comparison+legacy enabling' stamp: 'kph 11/12/2008 20:14'!
= aRioAble

	^ executive is: self sameAs: aRioAble asFile! !

!FileKernel methodsFor: 'coercing' stamp: 'kph 11/12/2008 20:06'!
asDirectory

	^ self! !

!FileKernel methodsFor: 'coercing' stamp: 'kph 11/12/2008 20:05'!
asFile

	^ self! !

!FileKernel methodsFor: 'coercing' stamp: 'kph 11/2/2008 23:11'!
asPath

	^ self! !

!FileKernel methodsFor: 'comparison+legacy enabling' stamp: 'kph 3/30/2007 05:18'!
asSqueakPathName

	"permits a rio to be handed right into open:forWrite:"

	^ self full value! !

!FileKernel methodsFor: 'coercing' stamp: 'kph 3/30/2007 03:10'!
asString

^ value! !

!FileKernel methodsFor: 'comparison+legacy enabling' stamp: 'kph 3/30/2007 05:18'!
asVmPathName

	"permits a rio to be handed right into open:forWrite:"

	^ self full value ! !

!FileKernel methodsFor: 'comparison+legacy enabling' stamp: 'kph 11/2/2008 23:11'!
at: n
	"this is purely to enable Rio's to be concatenated with strings" 

	^ self value at: n! !

!FileKernel methodsFor: 'accessing' stamp: 'kph 3/21/2007 23:41'!
base: newBase version: newVersion ext: newExt

	"only changes non nil values"
	
	| base version ext |
	
	self splitToBaseVersionAndExt: [ :b :v :e |
		base 	:= newBase 		ifNil: [ b ].
		version := newVersion 	ifNil: [ v ].
		ext 		:= newExt		ifNil: [ e ].
	].
		
	self fileName: 
		(String streamContents: [ :str | 
			str nextPutAll: base. 
			version = 0 ifFalse: [ 
				str nextPut: $. ; nextPutAll: version asString 
			].
			ext notEmpty ifTrue: [ 
				str nextPut: $.; nextPutAll: ext 
			]
		]). 
! !

!FileKernel methodsFor: 'public file' stamp: 'kph 11/6/2008 14:48'!
basicReader
 	
	^ executive basicReader: self! !

!FileKernel methodsFor: 'public file' stamp: 'kph 11/6/2008 14:43'!
basicWriter

	^ executive basicWriter: self! !

!FileKernel methodsFor: 'file copy' stamp: 'kph 11/12/2008 20:14'!
copyTo: aPathOrFile

	| in out |
	
	self validateIsFile.
	 
	(in := self reader) binary copyTo: (out := aPathOrFile asFile writer) binary.
	out close.
	in close.
	
	^ aPathOrFile asFile
! !

!FileKernel methodsFor: 'enumeration' stamp: 'kph 3/15/2007 04:43'!
entries

	^ self select: [:e | true ]! !

!FileKernel methodsFor: 'accessing' stamp: 'kph 4/3/2007 13:10'!
executive 
 	 
	^ executive  ! !

!FileKernel methodsFor: 'accessing' stamp: 'kph 3/26/2007 19:59'!
executive: aSystem

 	 executive := aSystem! !

!FileKernel methodsFor: 'testing' stamp: 'kph 11/6/2008 02:06'!
exists

^ self isDirectory or: [ self isFile ]! !

!FileKernel methodsFor: 'accessing' stamp: 'test 11/5/2008 23:33'!
fileName

 	^ stat ifNotNil: [ stat fileName ] ifNil: [  self splitToPathAndName: [ :path :n |  n ] ]
! !

!FileKernel methodsFor: 'accessing' stamp: 'kph 3/30/2007 03:23'!
fileName: newFileName
 
	self splitToPathAndName: [ :dirPath :fileName | 
		value := executive importValue: ((self newFrom: dirPath) pathJoin: newFileName) asString
 	].! !

!FileKernel methodsFor: 'comparison+legacy enabling' stamp: 'kph 11/2/2008 23:11'!
findTokens: t

	"this method enables Rio to be passed into a MultiByteFileStream just as a string, it is not needed in 3.9+"

	^ self asString findTokens: t! !

!FileKernel methodsFor: 'accessing' stamp: 'kph 3/30/2007 04:01'!
full
	
	^ executive fullFor: self! !

!FileKernel methodsFor: 'as yet unclassified' stamp: 'kph 11/2/2008 23:10'!
hash

	^ executive hashFor: self! !

!FileKernel methodsFor: 'testing' stamp: 'kph 4/11/2007 06:24'!
isDirectory

	^ executive isDirectory: self ! !

!FileKernel methodsFor: 'coercing' stamp: 'kph 3/23/2007 05:42'!
isEmpty

^ value isEmpty! !

!FileKernel methodsFor: 'testing' stamp: 'kph 11/3/2008 01:04'!
isFile

	^ executive isFile: self! !

!FileKernel methodsFor: 'testing' stamp: 'kph 11/12/2008 23:42'!
isFileKernel

	^true! !

!FileKernel methodsFor: 'testing' stamp: 'kph 11/12/2008 23:44'!
isFileKernelInstance

	^true! !

!FileKernel methodsFor: 'testing' stamp: 'kph 3/15/2007 04:40'!
isRecursive

	^ false! !

!FileKernel methodsFor: 'testing' stamp: 'kph 3/31/2007 02:46'!
isRoot

	^ executive isRoot: self! !

!FileKernel methodsFor: 'versions' stamp: 'kph 3/21/2007 23:44'!
latestVersion

^ self copy base: nil version: (self versions last) ext: nil! !

!FileKernel methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 00:13'!
newFrom: pathOrFileOrDirectory

	^ self class executive: executive value: pathOrFileOrDirectory! !

!FileKernel methodsFor: 'versions' stamp: 'kph 9/28/2007 00:45'!
nextVersion

	"the next non-exisiting version (self if none)"
	
^ self copy base: nil version: ((self versions ifEmpty: [ ^ self ]) last + 1) ext: nil! !

!FileKernel methodsFor: 'accessing' stamp: 'kph 11/13/2008 00:44'!
parent

	"note that the parent of root is the file system 'executive' "
	
	^self isRoot ifTrue: [ executive ]
 				ifFalse: [ executive dirClass executive: executive value: self full path ]
	! !

!FileKernel methodsFor: 'accessing' stamp: 'kph 3/23/2007 03:23'!
path

	^ self splitToPathAndName: [ :path :n |  path ].! !

!FileKernel methodsFor: 'private' stamp: 'kph 4/1/2007 16:32'!
pathJoin: morePath

	 (executive isRoot: value) ifTrue: [ ^ value , morePath asString ].
	
	 ^value isEmpty ifTrue: [ morePath asString ]
					ifFalse: [ self value, executive pathDelimiter, morePath asString ].
! !

!FileKernel methodsFor: 'printing' stamp: 'kph 11/2/2008 23:52'!
printOn: str

	str << $( << self class name << ' new: ''' << value << ''')'
	
! !

!FileKernel methodsFor: 'public file' stamp: 'kph 3/21/2007 04:21'!
reader
 	
	^ self basicReader! !

!FileKernel methodsFor: 'testing' stamp: 'kph 11/12/2008 20:15'!
sameAs: aFileAble

	^ executive is: self sameAs: aFileAble asFile! !

!FileKernel methodsFor: 'enumeration' stamp: 'kph 11/7/2008 03:28'!
select: selectBlock

	"select: returns an empty result if the rio is not a valid directory."

	^ executive in: self select: selectBlock! !

!FileKernel methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2008 00:42'!
setExecutive: e value: aRef  

	"create a fully initialized rio"

	executive := e.
	self value: aRef.
	
	^ self ! !

!FileKernel methodsFor: 'as yet unclassified' stamp: 'kph 3/30/2007 01:56'!
setMacType: typeString creator: creatorString
 	"Set the Macintosh file type and creator info for the file with the given name. Fails if the file does not exist or if the type and creator type arguments are not strings of length 4."
	 
 	executive set: self macType: typeString creator: creatorString! !

!FileKernel methodsFor: 'accessing' stamp: 'kph 11/23/2008 01:32'!
setStatFromDir: dir andEntryArray: a 
	
	stat := FileStat in: dir array: a
! !

!FileKernel methodsFor: 'as yet unclassified' stamp: 'kph 11/2/2008 23:52'!
size
	"size returns the path length, this is to enable Rio's to be concatenated with strings" 

	^ self value size! !

!FileKernel methodsFor: 'accessing' stamp: 'kph 3/21/2007 23:41'!
splitToBaseVersionAndExt: block

	| fileName version ext period |
	
	fileName := self fileName.
	period := fileName lastIndexOf: $..
	((period <=1) | (period = fileName size)) ifTrue: [ ^ block value: fileName value: 0 value: '' ].
	 
	ext := fileName copyFrom: (period + 1) to: fileName size.
 	fileName := fileName copyFrom: 1 to: period - 1.
	
	period := fileName lastIndexOf: $. ifAbsent: [ ^ block value: fileName value: 0 value: ext ].
	
	version := fileName copyFrom: period + 1 to: fileName size.
	version do: [ :char | char isDigit ifFalse: [ ^ block value: fileName value: 0 value: ext  ]].
		
	version := version asInteger.
	fileName := fileName copyFrom: 1 to: period - 1.
	
	^ block value: fileName value: version value: ext
	
! !

!FileKernel methodsFor: 'as yet unclassified' stamp: 'kph 4/1/2007 16:33'!
splitToPathAndName: pathAndNameBlock
 
	"Take the path and convert it to the path name of a directory and a local file name within that directory. FileName must be of the form: <dirPath><delimiter><localName>, where <dirPath><delimiter> is optional. The <dirPath> part may contain delimiters."

	| i dirName localName fullName |
	
	fullName := self value.
	i := fullName lastIndexOf: executive pathDelimiterChar.
	
	i == 0 ifTrue: [ ^ pathAndNameBlock value: '' value: fullName ].
	
	dirName := fullName copyFrom: 1 to: (i - 1 max: 1).
	localName := fullName copyFrom: i + 1 to: fullName size.

	^ pathAndNameBlock value: dirName value: localName! !

!FileKernel methodsFor: 'stat' stamp: 'kph 11/13/2008 00:24'!
stat

	| full |

	(stat notNil and: [ stat isValid ]) ifTrue: [ ^ stat ].

	full := self full.
	(Directory executive: executive value: full path) select: [ :e | 
		
		e = full ifTrue: [ ^ stat ifNil: [ e stat ] ifNotNil: [ stat copyFrom: e stat. ] ]. 
			
	false. 
	
	].
	
	^ stat := nil.
	! !

!FileKernel methodsFor: 'stat' stamp: 'kph 11/3/2008 00:47'!
statIsNowInvalid

	"All rios copyied from this one, share a stat instance if it exists. 
	Invalidating that instance means that this and all such derived rios, 
	will refresh their stat when needed."
	
	stat ifNotNil: [ stat invalidate ].

! !

!FileKernel methodsFor: 'validation & errors' stamp: 'kph 3/22/2007 04:32'!
validateIsDirectory

	self isDirectory ifFalse: [ ^self error: 'not a directory at: ', self value ].
	 ! !

!FileKernel methodsFor: 'validation & errors' stamp: 'kph 3/16/2007 10:17'!
validateIsFile

	self isFile ifFalse: [ ^self error: 'file not readable at: ', self value ].
	 ! !

!FileKernel methodsFor: 'private' stamp: 'kph 3/15/2007 02:34'!
value

	^ value ! !

!FileKernel methodsFor: 'as yet unclassified' stamp: 'kph 11/6/2008 00:21'!
value: aStringable
		
 	 value := executive importValue: aStringable asString.
	
! !

!FileKernel methodsFor: 'versions' stamp: 'kph 3/21/2007 23:42'!
versions

	| versions |

	versions := SortedCollection new.	
	self splitToBaseVersionAndExt:  [ :myB :myV :myE |
		self parent entries select: [ :each | 
			each splitToBaseVersionAndExt: [ :eachB :eachV :eachE |    			
				((eachB = myB) and: [ eachE = myE ]) ifTrue: [ versions add: eachV ]	.
			false.
			]
		]
	].

	^ versions! !

!FileKernel methodsFor: 'public file' stamp: 'kph 3/21/2007 04:21'!
writer

	^ self basicWriter! !

Object subclass: #FileStat
	instanceVariableNames: 'array dir'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'File-Kernel'!

!FileStat commentStamp: 'kph 3/8/2007 02:52' prior: 0!
A RioDirEntry is a wrapper for the results of primLookup to be more user friendly. For example modification times are avalable as squeak DateAndTime instances.
 !

!FileStat class methodsFor: 'as yet unclassified' stamp: 'kph 3/8/2007 02:48'!
in: dirRio array: theArray

	^ self new setDir: dirRio array: theArray! !

!FileStat methodsFor: 'as yet unclassified' stamp: 'kph 11/25/2008 20:49'!
creationTime

	^ TimeStamp fromSeconds: (array at: 2)! !

!FileStat methodsFor: 'as yet unclassified' stamp: 'kph 3/8/2007 02:49'!
dir

	^ dir! !

!FileStat methodsFor: 'as yet unclassified' stamp: 'kph 11/25/2008 20:49'!
fileName

 ^ array at: 1! !

!FileStat methodsFor: 'as yet unclassified' stamp: 'kph 11/25/2008 20:49'!
fileSize

	^ array at: 5! !

!FileStat methodsFor: 'as yet unclassified' stamp: 'kph 11/6/2008 17:10'!
invalidate
	
	array := nil.! !

!FileStat methodsFor: 'as yet unclassified' stamp: 'kph 11/25/2008 20:49'!
isDirectory

	^ array at: 4! !

!FileStat methodsFor: 'as yet unclassified' stamp: 'kph 11/25/2008 20:50'!
isFile

	^ self isDirectory not! !

!FileStat methodsFor: 'as yet unclassified' stamp: 'kph 11/25/2008 20:50'!
isInvalid

	^ array isNil! !

!FileStat methodsFor: 'as yet unclassified' stamp: 'kph 11/25/2008 20:50'!
isValid

	^ array notNil! !

!FileStat methodsFor: 'as yet unclassified' stamp: 'kph 11/25/2008 20:48'!
modificationTime

	^ TimeStamp fromSeconds: (array at: 3)

! !

!FileStat methodsFor: 'as yet unclassified' stamp: 'kph 11/12/2008 20:05'!
setDir: aDir array: data

	dir := aDir.
	array := data.! !

!PositionableStream methodsFor: '*file-kernel' stamp: 'kph 11/7/2008 00:51'!
copyTo: out
		
	| buffer first |
 	
	self atEnd ifTrue: [ ^ self ].
	
	first := self next.
	
	buffer := (first isCharacter 
				ifTrue: [ String ] 
				ifFalse: [ out binary. ByteArray ]) new: 50000.
	
	out nextPut: first.

	[self atEnd] whileFalse:
		[out nextPutAll: (self nextInto: buffer)].
! !

!PositionableStream methodsFor: '*file-kernel' stamp: 'kph 3/18/2007 01:45'!
size

	^ readLimit! !

!SocketStream methodsFor: '*file-kernel' stamp: 'kph 11/7/2008 01:46'!
copyTo: out
	
	self isBinary ifTrue: [ out binary ].
		
	[self atEnd] whileFalse:
		[ self receiveAvailableData.
		  out nextPutAll: self nextAllInBuffer.
		  ].
! !

!SocketStream methodsFor: '*file-kernel' stamp: 'kph 11/7/2008 01:47'!
copyTo: out size: size withProgress: label 

	| barPos read |
 	
	self isBinary ifTrue: [ out binary ].

	label asString displayProgressAt: Sensor cursorPoint
		from: (barPos := 0) to: size
		during: [:bar |
				[self atEnd] whileFalse:
					[ 
					  bar value: barPos.
					  self receiveAvailableData.
					  out nextPutAll: (read := self nextAllInBuffer).
					  barPos := barPos + read size
				  ].

		].! !

!Collection methodsFor: '*file-kernel-override' stamp: 'kph 9/28/2007 00:44'!
ifEmpty: aBlock
        self isEmpty ifTrue: [ ^ aBlock value ] ! !

!StandardFileStream class methodsFor: '*file-kernel-override' stamp: 'kph 3/13/2007 20:54'!
retryWithGC: execBlock until: testBlock forFileNamed: fullName
	"Re-implemented to only force GC if a file with the given name exists"
	| blockValue foundIt |
	blockValue := execBlock value.
	(testBlock value: blockValue) ifTrue:[^blockValue].
	"See if we have a file with the given name"
	foundIt := Registry keys "hold on strongly for now" 
		anySatisfy:[:file| file name sameAs: fullName asString ].
	foundIt ifFalse:[^blockValue].
	Smalltalk garbageCollectMost.
	blockValue := execBlock value.
	(testBlock value: blockValue) ifTrue:[^blockValue].
	Smalltalk garbageCollect.
	^execBlock value.! !

!StandardFileStream methodsFor: '*file-kernel-override' stamp: 'kph 3/9/2007 06:43'!
close
	"Close this file."

	fileID ifNotNil: [
		self primClose: fileID.
		self unregister.
		fileID := nil].
	
	^ name
! !
FileLocalExecutive initialize!
FileKernel initialize!
